/*****************************************************************

Copyright (c) 1996-2001 the kicker authors. See file AUTHORS.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************/

#include <qtooltip.h>

#include <klocale.h>
#include <kapplication.h>
#include <kdebug.h>

#include "config.h"

#include "kicker.h"
#include "k_mnu.h"

#include "kbutton.h"
#include "kbutton.moc"

#if defined(HAVE_XTEST)
#   define XK_MISCELLANY
#   include <X11/Xlib.h>
#   include <X11/keysymdef.h>
#   include <X11/extensions/XTest.h>

    static unsigned short g_keycodeForSuperL, g_keycodeForSuperR;
#endif

KButton::KButton( QWidget* parent )
    : PanelPopupButton( parent, "KButton" )
{
    QToolTip::add(this, i18n("Start applications"));
    setTitle(i18n("Start Applications"));

    setPopup( Kicker::kicker()->KMenu() );
    setIcon("kmenu");

#if defined(HAVE_XTEST)
    g_keycodeForSuperL = XKeysymToKeycode( qt_xdisplay(), XK_Super_L );
    g_keycodeForSuperR = XKeysymToKeycode( qt_xdisplay(), XK_Super_R );

    XKeyboardControl kbd;
    kbd.auto_repeat_mode = AutoRepeatModeOff;
    kbd.key = g_keycodeForSuperL;
    XChangeKeyboardControl( qt_xdisplay(), KBKey | KBAutoRepeatMode, &kbd );
    kbd.key = g_keycodeForSuperR;
    XChangeKeyboardControl( qt_xdisplay(), KBKey | KBAutoRepeatMode, &kbd );

    if( g_keycodeForSuperL || g_keycodeForSuperR ) {
        //kdDebug() << "g_keycodeForSuperL = " << g_keycodeForSuperL
        //    << ", g_keycodeForSuperR = " << g_keycodeForSuperR << endl;
        if( g_keycodeForSuperL )
            XGrabKey( qt_xdisplay(), g_keycodeForSuperL, 0, qt_xrootwin(), True, GrabModeAsync, GrabModeSync );
        if( g_keycodeForSuperR )
            XGrabKey( qt_xdisplay(), g_keycodeForSuperR, 0, qt_xrootwin(), True, GrabModeAsync, GrabModeSync );
        kapp->installX11EventFilter( this );
    }
#endif
}

void KButton::properties()
{
    KApplication::startServiceByDesktopName("kmenuedit", QStringList(),
                                            0, 0, 0, "", true);
}

void KButton::initPopup()
{
//    kdDebug(1210) << "KButton::initPopup()" << endl;

    // this hack is required to ensure the correct popup position
    // when the size of the recent application part of the menu changes
    // please don't remove this _again_
    Kicker::kicker()->KMenu()->initialize();
}

#if defined(HAVE_XTEST)
extern Time qt_x_time;

bool KButton::x11Event( XEvent* event )
{
    static bool s_bWatching = false;

    if (event->type == KeyPress) {
        //cout << "key pressed: window " << event->xkey.window << endl;

        // 115 = Super_L
        if (event->xkey.state == 0 && event->xkey.keycode &&
            (event->xkey.keycode == g_keycodeForSuperL ||
             event->xkey.keycode == g_keycodeForSuperR)) {
            XUngrabKeyboard( qt_xdisplay(), event->xkey.time );
            if( XGrabKeyboard(qt_xdisplay(), qt_xrootwin(), FALSE, GrabModeAsync, GrabModeAsync, qt_x_time) == GrabSuccess ) {
                s_bWatching = true;
                //cout << "grab!" << endl;
                return true;
            }
        }
        else if (s_bWatching) {
            XUngrabKeyboard( qt_xdisplay(), event->xkey.time );
            XTestFakeKeyEvent( qt_xdisplay(), event->xkey.keycode, true, 0 );
            //cout << "sent" << endl;
            s_bWatching = false;
        }
    }

    else if (event->type == KeyRelease && s_bWatching) {
        XUngrabKeyboard( qt_xdisplay(), event->xkey.time );
        //releaseKeyboard();
        if (s_bWatching) {
            if (event->xkey.keycode &&
                (event->xkey.keycode == g_keycodeForSuperL ||
                 event->xkey.keycode == g_keycodeForSuperR))
            {
                //cout << "Win released! " << popup()->isVisible() << endl;
                if (popup()->isVisible())
                    popup()->hide();
                else
                    slotExecMenu();
            }
            s_bWatching = false;
        }
    }

    return false;
}
#else
bool KButton::x11Event( XEvent* )
{
    return false;
}
#endif
