/***************************************************************************
 *   Copyright Brian Ledbetter 2001-2003 <brian@shadowcom.net>             *
 *   Copyright Ravikiran Rajagopal 2003                                    *
 *   ravi@ee.eng.ohio-state.edu                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License (version 2) as   *
 *   published by the Free Software Foundation. (The original KSplash/ML   *
 *   codebase (upto version 0.95.3) is BSD-licensed.)                      *
 *                                                                         *
 ***************************************************************************/

#ifndef __OBJKSTHEME_H__
#define __OBJKSTHEME_H__

#include <kcmdlineargs.h>

#include <qstring.h>
#include <qstringlist.h>

class KConfig;
/**
 * @short Theme reader.
 * A Theme is read in from the file
 * <KDEDIR>/share/apps/ksplashml/Themes/<theme>/theme.rc
 * This controls the behavior, graphics, and appearance
 * of KSplash completely, and offers a friendlier way
 * of installing custom splash screens.
 */
class ObjKsTheme : public QObject
{
  Q_OBJECT
public:
  ObjKsTheme( const QString& );

  void loadCmdLineArgs( KCmdLineArgs * );

  inline QString theme()       { return( mActiveTheme ); }
  inline QString themeEngine() { return( mThemeEngine ); }
  inline KConfig *themeConfig(){ return( mThemeConfig ); }
  inline QString themeDir()    { return( mThemeDir );    }
  inline bool loColor()        { return( mLoColor );     }
  inline bool testing()        { return( mTesting );     }
  inline bool managedMode()    { return( mManagedMode ); }
  inline QString icon( int i ) { return (m_icons[i-1].isNull()?(QString::null):m_icons[i-1]); }
  inline QString text( int i ) { return (m_text[i-1].isNull()?(QString::null):m_text[i-1]);   }

protected:
  bool loadThemeRc( const QString&, bool );
  bool loadLocalConfig( const QString&, bool );
  bool loadKConfig( KConfig *, const QString&, bool );

private:
  QString mActiveTheme, mThemeDir;
  KConfig *mThemeConfig;


  bool mLoColor, mTesting, mManagedMode;
  QString mThemeEngine;

  QStringList m_icons, m_text;
};

#endif
